from rest_framework import serializers

from netbox.api.serializers import NetBoxModelSerializer
from netbox_storage.api.nested_serializers import NestedFilesystemSerializer, NestedDriveSerializer, \
    NestedLinuxVolumeSerializer
from netbox_storage.models import Drive, Filesystem, LinuxVolume, LinuxVolumeDrive
from virtualization.api.nested_serializers import NestedClusterSerializer, NestedVirtualMachineSerializer


class FilesystemSerializer(serializers.ModelSerializer):
    class Meta:
        model = Filesystem
        fields = (
            "id",
            "filesystem",
            "description",
            "created",
            "last_updated",
            "custom_fields",
        )


class DriveSerializer(NetBoxModelSerializer):
    cluster = NestedClusterSerializer(required=False, allow_null=True)
    virtual_machine = NestedVirtualMachineSerializer(required=False, allow_null=True)
    url = serializers.HyperlinkedIdentityField(view_name="plugins-api:netbox_storage-api:drive-detail")

    class Meta:
        model = Drive
        fields = (
            "id",
            "url",
            "display",
            "size",
            "cluster",
            "virtual_machine",
            "identifier",
            "description",
            "created",
            "last_updated",
            "custom_fields",
        )


class LinuxVolumeSerializer(serializers.ModelSerializer):
    fs = NestedFilesystemSerializer(required=False, allow_null=True)

    class Meta:
        model = LinuxVolume
        fields = (
            "id",
            "vg_name",
            "lv_name",
            "fs",
            "path",
            "description",
            "created",
            "last_updated",
            "custom_fields",
        )


class LinuxVolumeDriveSerializer(serializers.ModelSerializer):
    drive = NestedDriveSerializer(required=False, allow_null=True)
    linuxvolume = NestedLinuxVolumeSerializer(required=False, allow_null=True)

    class Meta:
        model = LinuxVolumeDrive
        fields = (
            "id",
            "allocation",
            "drive",
            "linuxvolume",
            "description",
            "created",
            "last_updated",
            "custom_fields",
        )



from django.db.models import Q

from netbox.filtersets import NetBoxModelFilterSet

from netbox_storage.models import LinuxVolume, LinuxVolumeDrive


class LinuxVolumeDriveFilter(NetBoxModelFilterSet):
    """Filter capabilities for LinuxVolume instances."""

    class Meta:
        model = LinuxVolumeDrive
        fields = [
            "drive",
            "linuxvolume",
            "allocation",
            "description",
        ]

    def search(self, queryset, name, value):
        """Perform the filtered search."""
        if not value.strip():
            return queryset
        qs_filter = (
            Q(allocation__icontains=value)
        )
        return queryset.filter(qs_filter)



from django.core.validators import MinValueValidator
from django.forms import (
    CharField, IntegerField,
)

from netbox.forms import (
    NetBoxModelBulkEditForm,
    NetBoxModelFilterSetForm,
    NetBoxModelImportForm,
    NetBoxModelForm,
)
from utilities.forms import (
    DynamicModelChoiceField,
    APISelect,
)

from django.urls import reverse_lazy

from netbox_storage.models import Filesystem, LinuxVolumeDrive, Drive, LinuxVolume


class LinuxVolumeDriveForm(NetBoxModelForm):
    """Form for creating a new LinuxVolume object."""

    allocation = IntegerField(
        required=True,
        label="Size (GB)",
        help_text="The size of the drive e.g. 25",
        validators=[MinValueValidator(1)],
    )
    linuxvolume = DynamicModelChoiceField(
        queryset=LinuxVolume.objects.all(),
        label="Linuxvolume",
        widget=APISelect(
            attrs={"data-url": reverse_lazy("plugins-api:netbox_storage-api:linuxvolume-list")}
        ),
        help_text="The linuxvolume e.g. docker",
    )
    drive = DynamicModelChoiceField(
        queryset=Drive.objects.all(),
        label="Drive",
        widget=APISelect(
            attrs={"data-url": reverse_lazy("plugins-api:netbox_storage-api:drive-list")}
        ),
        help_text="The Filesystem of the Volume e.g. ext4",
    )

    class Meta:
        model = LinuxVolumeDrive

        fields = (
            "drive",
            "linuxvolume",
            "allocation",
            "description",
        )


class LinuxVolumeDriveFilterForm(NetBoxModelFilterSetForm):
    """Form for filtering LinuxVolume instances."""

    model = LinuxVolumeDrive

    allocation = IntegerField(
        required=True,
        label="Size (GB)",
        help_text="The size of the drive e.g. 25",
        validators=[MinValueValidator(1)],
    )
    linuxvolume = DynamicModelChoiceField(
        queryset=LinuxVolume.objects.all(),
        label="Filesystem Name",
        widget=APISelect(
            attrs={"data-url": reverse_lazy("plugins-api:netbox_storage-api:filesystem-list")}
        ),
        help_text="The Filesystem of the Volume e.g. ext4",
    )
    drive = DynamicModelChoiceField(
        queryset=Drive.objects.all(),
        label="Filesystem Name",
        widget=APISelect(
            attrs={"data-url": reverse_lazy("plugins-api:netbox_storage-api:filesystem-list")}
        ),
        help_text="The Filesystem of the Volume e.g. ext4",
    )


class LinuxVolumeDriveImportForm(NetBoxModelImportForm):
    fs = DynamicModelChoiceField(
        queryset=Filesystem.objects.all(),
        required=False
    )

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

    class Meta:
        model = LinuxVolumeDrive

        fields = (
            "drive",
            "linuxvolume",
            "allocation",
            "description",
        )


class LinuxVolumeDriveBulkEditForm(NetBoxModelBulkEditForm):
    model = LinuxVolumeDrive

    allocation = IntegerField(
        required=True,
        label="Size (GB)",
        help_text="The size of the drive e.g. 25",
        validators=[MinValueValidator(1)],
    )
    linuxvolume = DynamicModelChoiceField(
        queryset=LinuxVolume.objects.all(),
        label="Filesystem Name",
        widget=APISelect(
            attrs={"data-url": reverse_lazy("plugins-api:netbox_storage-api:filesystem-list")}
        ),
        help_text="The Filesystem of the Volume e.g. ext4",
    )
    drive = DynamicModelChoiceField(
        queryset=Drive.objects.all(),
        label="Filesystem Name",
        widget=APISelect(
            attrs={"data-url": reverse_lazy("plugins-api:netbox_storage-api:filesystem-list")}
        ),
        help_text="The Filesystem of the Volume e.g. ext4",
    )
    description = CharField(max_length=255, required=False)

    fieldsets = (
        (
            None,
            ("drive", "linuxvolume", "allocation", "description",),
        ),
    )
    nullable_fields = ["description"]



from netbox.views import generic

from netbox_storage.filters import LinuxVolumeDriveFilter
from netbox_storage.forms import (
    LinuxVolumeDriveImportForm,
    LinuxVolumeDriveFilterForm,
    LinuxVolumeDriveForm,
    LinuxVolumeDriveBulkEditForm
)

from netbox_storage.models import LinuxVolumeDrive
from netbox_storage.tables import LinuxVolumeDriveTable


class LinuxVolumeDriveListView(generic.ObjectListView):
    queryset = LinuxVolumeDrive.objects.all()
    filterset = LinuxVolumeDriveFilter
    filterset_form = LinuxVolumeDriveFilterForm
    table = LinuxVolumeDriveTable


class LinuxVolumeDriveView(generic.ObjectView):
    """Display LinuxVolume details"""

    queryset = LinuxVolumeDrive.objects.all()


class LinuxVolumeDriveEditView(generic.ObjectEditView):
    """View for editing a LinuxVolumeDrive instance."""

    queryset = LinuxVolumeDrive.objects.all()
    form = LinuxVolumeDriveForm
    default_return_url = "plugins:netbox_storage:linuxvolumedrive_list"


class LinuxVolumeDriveDeleteView(generic.ObjectDeleteView):
    queryset = LinuxVolumeDrive.objects.all()
    default_return_url = "plugins:netbox_storage:linuxvolumedrive_list"


class LinuxVolumeDriveBulkImportView(generic.BulkImportView):
    queryset = LinuxVolumeDrive.objects.all()
    model_form = LinuxVolumeDriveImportForm
    table = LinuxVolumeDriveTable
    default_return_url = "plugins:netbox_storage:linuxvolumedrive_list"


class LinuxVolumeDriveBulkEditView(generic.BulkEditView):
    queryset = LinuxVolumeDrive.objects.all()
    filterset = LinuxVolumeDriveFilter
    table = LinuxVolumeDriveTable
    form = LinuxVolumeDriveBulkEditForm


class LinuxVolumeDriveBulkDeleteView(generic.BulkDeleteView):
    queryset = LinuxVolumeDrive.objects.all()
    table = LinuxVolumeDriveTable




import django_tables2 as tables

from netbox.tables import (
    NetBoxTable,
    ToggleColumn,
    ActionsColumn,
)

from netbox_storage.models import LinuxVolumeDrive


class LinuxVolumeDriveBaseTable(NetBoxTable):
    """Base class for tables displaying LinuxVolume"""

    drive = tables.Column(
        linkify=True,
        verbose_name="drive"
    )
    linuxvolume = tables.Column(
        linkify=True,
        verbose_name="linuxvolume"
    )
    allocation = tables.Column(
        linkify=True,
        verbose_name="allocation"
    )


class LinuxVolumeDriveTable(LinuxVolumeDriveBaseTable):
    """Table for displaying LinuxVolume objects."""

    pk = ToggleColumn()

    class Meta(NetBoxTable.Meta):
        model = LinuxVolumeDrive
        fields = (
            "pk",
            "drive",
            "linuxvolume",
            "allocation",
            "description",
        )
        default_columns = (
            "drive",
            "linuxvolume",
            "allocation",
            "description"
        )


class RelatedLinuxVolumeTable(LinuxVolumeDriveBaseTable):
    actions = ActionsColumn(actions=())

    class Meta(NetBoxTable.Meta):
        model = LinuxVolumeDrive
        fields = (
            "pk",
            "drive",
            "linuxvolume",
            "allocation",
            "description",
        )
        default_columns = (
            "drive",
            "linuxvolume",
            "allocation",
            "description"
        )

